c chi2_func_SYN.f
c Calculate chi^2 for synthetic spectra.
c Miroslav Broz (miroslav.broz@email.cz), May 23rd 2016

      subroutine chi2_func_SYN(NOUT, tout, vb, chi2, n)

      implicit none
      include '../misc/const.inc'
      include 'simplex.inc'
      include 'dependent.inc'

c input
      integer NOUT
      real*8 tout(OUTMAX)
      real*8 vb(OUTMAX,NBODMAX,3)

c output
      real*8 chi2
      integer n

c observational data
      integer m_OBS
      real*8 t_OBS(OBSMAX), lambda_OBS(OBSMAX), Int_OBS(OBSMAX),
     :  sigma_Int_OBS(OBSMAX)
      integer dataset_OBS(OBSMAX)

c synthetic spectra
      integer n_synth(NBODMAX)
      real*8 lambda_synth(OBSMAX,NBODMAX), Int_synth(OBSMAX,NBODMAX)

c internal variables
      integer i,j,k,l,i1st,iu,ier
      real*8 chi2_
      real*8 vzb_interp, lambda_interp, Int_interp, Int_Sum
      real*8 Lum_, Lumtot

c external functions
      real*8 interp, Lum_func
      real*8 T, R
      common /cb_Lum/ T, R

      data i1st /0/
      data m_OBS /0/
      data iu /10/

      save i1st,
     :  m_OBS, t_OBS, lambda_OBS, Int_OBS, sigma_Int_OBS, dataset_OBS,
     :  n_synth, lambda_synth, Int_synth

c-----------------------------------------------------------------------
c
c read synthetic and observed spectra (only 1st time!)
c

      if (i1st.eq.0) then

        do j = 1, nbod

          call read_synth(file_synth(j), n_synth(j), lambda_synth(1,j),
     :      Int_synth(1,j))

          if (debug) then
            write(*,*) "# n_synth(", j, ") = ", n_synth(j)
          endif

        enddo

        call read_SYN(file_SYN, m_OBS, t_OBS, Int_OBS, sigma_Int_OBS,
     :    dataset_OBS)

        if (debug) then
          write(*,*) "# m_SYN = ", m_OBS
        endif

        lambda1 = lambda1

        i1st = 1

      endif  ! i1st

c-----------------------------------------------------------------------
c
c chi^2 for synthetic spectra
c
      chi2 = 0.d0
      n = 0

      if (debug) then
        open(unit=iu,file="chi2_SYN.dat",status="unknown")
        write(iu,*) "# t_OBS & lambda & Int_interp & sigma_OBS",
     :    " & dataset & chi^2"
        write(iu,*) "# t_OBS & lambda & Int_OBS    & sigma_OBS",
     :    " & dataset & chi^2"
      endif

      j = 2
      do i = 1, m_OBS

        if ((lambda_OBS(i).gt.lambda1).and.(lambda_OBS(i).lt.lambda2))
     :    then
    
          do while ((j.lt.NOUT).and.(tout(j).le.t_OBS(i)))
            j = j+1
          enddo

          Int_sum = 0.d0

          do k = 1, nbod

c linear interpolation of integrated data to a given position in time

            vzb_interp = interp(tout(j-1), tout(j), vb(j-1,k,3),
     :        vb(j,k,3), t_OBS(i))  ! k-th body, vb_z coordinate

c interpolate synthetic intensities to given lambda using Hermite polynomials

            lambda_interp = lambda_OBS(i) * (1.d0 - vzb_interp/c)   ! Doppler shift

            call intep(lambda_interp, Int_interp,
     :        lambda_synth(1,k), Int_synth(1,k), n_synth(k), IER)

            if (IER.eq.2) then
              write(*,*) "# Warning: IER.eq.2 in intep()"
            endif

c compute monochromatic "luminosity" at given lambda
            T = T_eff(k)
            R = R_star(k)*R_S
            Lum_ = Lum_func(lambda_interp)
            Lumtot = Lumtot + Lum_

            Int_sum = Int_sum + Int_interp*Lum_

          enddo  ! nbod

c normalize the intensity again
          Int_sum = Int_sum/Lumtot

c add to chi^2
          chi2_ = (Int_OBS(i) - Int_sum)**2/sigma_Int_OBS(i)**2
          chi2 = chi2 + chi2_
          n = n + 1

          if (debug) then
            write(iu,*) t_OBS(i), lambda_OBS(i), Int_sum,
     :        sigma_Int_OBS(i), dataset_OBS(i), chi2_
            write(iu,*) t_OBS(i), lambda_OBS(i), Int_OBS(i),
     :        sigma_Int_OBS(i), dataset_OBS(i), chi2_
            write(iu,*)
          endif

        endif  ! lambda

      enddo

      if (debug) then
        close(iu)
      endif

      return
      end


